/**
 * Copyright (c) 2017-2019, lt 北京中世康恺科技有限公司 (www.pacsonline.cn).
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 */

package com.zskk.common;

import com.jfinal.config.Constants;
import com.jfinal.config.Handlers;
import com.jfinal.config.Interceptors;
import com.jfinal.config.JFinalConfig;
import com.jfinal.config.Plugins;
import com.jfinal.config.Routes;
import com.jfinal.core.JFinal;
import com.jfinal.kit.PropKit;
import com.jfinal.plugin.activerecord.ActiveRecordPlugin;
import com.jfinal.plugin.druid.DruidPlugin;
import com.jfinal.plugin.redis.RedisPlugin;
import com.jfinal.plugin.redis.serializer.JdkSerializer;
import com.jfinal.template.Engine;
import com.jfinal.weixin.sdk.api.ApiConfig;
import com.jfinal.weixin.sdk.api.ApiConfigKit;
import com.jfinal.weixin.sdk.cache.RedisAccessTokenCache;
import com.zskk.service.ServiceFactory;
import com.zskk.controller.WeixinApiController;
import com.zskk.controller.WeixinMsgController;
import com.zskk.controller.WeixinPayController;
import com.zskk.model._MappingKit;
import com.zskk.controller.WebBridgeController;

public class ZskkConfig extends JFinalConfig {
	// 本地开发模式
	private boolean isLocalDev = false;

	/**
	 * 如果生产环境配置文件存在，则优先加载该配置，否则加载开发环境配置文件
	 * 
	 * @param pro 生产环境配置文件
	 * @param dev 开发环境配置文件
	 */
	public void loadProp(String pro, String dev) {
		try {
			PropKit.use(pro);
		} catch (Exception e) {
			PropKit.use(dev);
			isLocalDev = true;
		}
	}

	public void configConstant(Constants me) {
		loadProp("config.properties", "a_little_config.txt");
		me.setDevMode(PropKit.getBoolean("devMode", false));

		// ApiConfigKit 设为开发模式可以在开发阶段输出请求交互的 xml 与 json 数据
		ApiConfigKit.setDevMode(me.getDevMode());
	}

	public void configRoute(Routes me) {
		/**
		 * jfinal 3.6 新添加的配置项，如果有控制器继承了 MsgController 就必须 要添加下面的配置，该配置才能将超类
		 * MsgController 中的 index() 方法 映射为 action
		 * 
		 * 使用 jfinal 3.6 之前的版本不必理会这项配置
		 */
		me.setMappingSuperClass(true);

		me.add("/msg", WeixinMsgController.class);
		me.add("/api", WeixinApiController.class, "/api");
		me.add("/web", WebBridgeController.class);
		me.add("/pay", WeixinPayController.class, "/pay");

	}

	public void configPlugin(Plugins me) {

		DruidPlugin druidPlugin = new DruidPlugin(PropKit.get("jdbcUrl"), PropKit.get("user"),
				PropKit.get("password").trim());
		me.add(druidPlugin);

		ActiveRecordPlugin arp = new ActiveRecordPlugin(druidPlugin);
		me.add(arp);

		// 所有配置在 MappingKit 中搞定
		_MappingKit.mapping(arp);

		// 使用redis分布accessToken
		RedisPlugin redisPlugin = new RedisPlugin("weixin_patient", "127.0.0.1", 6379);
		redisPlugin.setSerializer(JdkSerializer.me); // 需要使用fst高性能序列化的用户请删除这一行（Fst jar依赖请查看WIKI）
		me.add(redisPlugin);
	}

	public void configInterceptor(Interceptors me) {
		// 设置默认的 appId 规则，默认值为appId，可采用url挂参数 ?appId=xxx 切换多公众号
//         ApiInterceptor.setAppIdParser(new AppIdParser.DefaultParameterAppIdParser("appId")); 默认无需设置
//         MsgInterceptor.setAppIdParser(new AppIdParser.DefaultParameterAppIdParser("appId")); 默认无需设置
	}

	public void configHandler(Handlers me) {

	}

	@Override
	public void onStart() {
		// TODO Auto-generated method stub
		super.onStart();
		// 支持redis存储access_token、js_ticket，需要先启动RedisPlugin
		ApiConfigKit.setAccessTokenCache(new RedisAccessTokenCache("weixin_patient"));
		// 中世康恺电子胶片平台
		ApiConfig ac_zskk = new ApiConfig();
		// 配置微信 API 相关参数
		ac_zskk.setToken(PropKit.get("token_zskk"));
		ac_zskk.setAppId(PropKit.get("appId_zskk"));
		ac_zskk.setAppSecret(PropKit.get("appSecret_zskk"));

		/**
		 * 是否对消息进行加密，对应于微信平台的消息加解密方式： 1：true进行加密且必须配置 encodingAesKey
		 * 2：false采用明文模式，同时也支持混合模式
		 */
		ac_zskk.setEncryptMessage(Boolean.FALSE);

		/**
		 * 多个公众号时，重复调用ApiConfigKit.putApiConfig(ac)依次添加即可，第一个添加的是默认。
		 */
		// 黑龙江省电子胶片平台
		ApiConfigKit.putApiConfig(ac_zskk);
		ApiConfig ac_hlj = new ApiConfig();
		ac_hlj.setToken(PropKit.get("token_hlj"));
		ac_hlj.setAppId(PropKit.get("appId_hlj"));
		ac_hlj.setAppSecret(PropKit.get("appSecret_hlj"));
		ac_hlj.setEncryptMessage(Boolean.FALSE);
		ApiConfigKit.putApiConfig(ac_hlj);
		
		ApiConfigKit.putApiConfig(ac_zskk);

		// guanying
		ApiConfig ac_zskk2 = new ApiConfig();
		ac_zskk2.setToken(PropKit.get("token_zskk2"));
		ac_zskk2.setAppId(PropKit.get("appId_zskk2"));
		ac_zskk2.setAppSecret(PropKit.get("appSecret_zskk2"));
		ac_zskk2.setEncryptMessage(Boolean.FALSE);
		ApiConfigKit.putApiConfig(ac_zskk2);

		// 辽宁省电子胶片平台
		ApiConfig ac_ln = new ApiConfig();
		ac_ln.setToken(PropKit.get("token_ln"));
		ac_ln.setAppId(PropKit.get("appId_ln"));
		ac_ln.setAppSecret(PropKit.get("appSecret_ln"));
		ac_ln.setEncryptMessage(Boolean.FALSE);
		ApiConfigKit.putApiConfig(ac_ln);

		// 河南省电子胶片平台
		ApiConfig ac_hn = new ApiConfig();
		ac_hn.setToken(PropKit.get("token_hn"));
		ac_hn.setAppId(PropKit.get("appId_hn"));
		ac_hn.setAppSecret(PropKit.get("appSecret_hn"));
		ac_hn.setEncryptMessage(Boolean.FALSE);
		ApiConfigKit.putApiConfig(ac_hn);

		// ceshi
//		ApiConfig ac_cs = new ApiConfig();
//		ac_cs.setToken(PropKit.get("token_cs"));
//		ac_cs.setAppId(PropKit.get("appId_cs"));
//		ac_cs.setAppSecret(PropKit.get("appSecret_cs"));
//		ac_ln.setEncryptMessage(Boolean.FALSE);
//		ApiConfigKit.putApiConfig(ac_cs);

		/**
		 * 1.9 新增LocalTestTokenCache用于本地和线上同时使用一套appId时避免本地将线上AccessToken冲掉
		 * 
		 * 设计初衷：https://www.oschina.net/question/2702126_2237352
		 * 
		 * 注意： 1. 上线时应保证此处isLocalDev为false，或者注释掉该不分代码！
		 * 
		 * 2. 为了安全起见，此处可以自己添加密钥之类的参数，例如：
		 * http://localhost/weixin/api/getToken?secret=xxxx
		 * 然后在WeixinApiController#getToken()方法中判断secret
		 * 
		 * @see WeixinApiController#getToken()
		 */
//      if (isLocalDev) {
//          String onLineTokenUrl = "http://localhost/weixin/api/getToken";
//          ApiConfigKit.setAccessTokenCache(new LocalTestTokenCache(onLineTokenUrl));
//      }
		ServiceFactory.init();

	}


//	public static void main(String[] args) {
//		JFinal.start("src/main/webapp", 10000, "/", 5);
//	}

	@Override
	public void configEngine(Engine engine) {

	}
}
