package com.zskk.controller;

import com.jfinal.kit.PropKit;
import com.jfinal.weixin.sdk.api.AccessTokenApi;
import com.jfinal.weixin.sdk.api.ApiConfigKit;
import com.jfinal.weixin.sdk.api.ApiResult;
import com.jfinal.weixin.sdk.api.CallbackIpApi;
import com.jfinal.weixin.sdk.api.CustomServiceApi;
import com.jfinal.weixin.sdk.api.MenuApi;
import com.jfinal.weixin.sdk.api.QrcodeApi;
import com.jfinal.weixin.sdk.api.ShorturlApi;
import com.jfinal.weixin.sdk.api.TemplateMsgApi;
import com.jfinal.weixin.sdk.api.UserApi;
import com.jfinal.weixin.sdk.jfinal.ApiController;
import com.zskk.model.Exams;
import com.zskk.model.Institution;
import com.zskk.model.Report;
import com.zskk.service.ServiceFactory;
import com.zskk.service.WeixinService;
import com.zskk.service.bean.WxJSConfigBean;

public class WeixinApiController extends ApiController {

    /**
	 * 为WeixinConfig onLineTokenUrl处提供AccessToken
	 * 
	 * 此处是为了开发测试和生产环境同时使用一套appId时为开发测试环境提供AccessToken
	 * 
	 * 设计初衷：https://www.oschina.net/question/2702126_2237352
	 */
	public void getToken() {
		try {
			String json = AccessTokenApi.getAccessToken().getAccessToken();
			renderText(json);
		} catch (Exception e) {
			// TODO: handle exception
			renderText(e.toString());
		}
	}

    /**
     * 获取公众号菜单
     */
    public void getMenu() {
        ApiResult apiResult = MenuApi.getMenu();
        if (apiResult.isSucceed())
            renderText(apiResult.getJson());
        else
            renderText(apiResult.getErrorMsg());
    }

    /**
     * 创建菜单
     */
    public void createMenu()
    {
        String str = "{\n" +
                "    \"button\": [\n" +
                "        {\n" +
                "            \"name\": \"扫一扫\",\n" +
                "            \"key\": \"rselfmenu_0_1\",\n" +
                "            \"type\": \"scancode_push\"\n" +
                "        },\n" +
                "        {\n" +
                "            \"name\": \"我的检查\",\n" +
                "            \"sub_button\": [\n" +
                "        {\n" +
                "\t    \"name\": \"我的检查列表\",\n" +
                "\t    \"url\": \"http://wechat.pacsonline.cn/wx_patient/web/examList?appId=" + ApiConfigKit.getApiConfig().getAppId() +"\",\n" +
                "\t    \"type\": \"view\"\n" +
                "        },\n" +
                "        {\n" +
                "\t    \"name\": \"添加绑定\",\n" +
                "\t    \"url\": \"http://wechat.pacsonline.cn/wx_patient/web/addBinding?appId=" + ApiConfigKit.getApiConfig().getAppId() +"\",\n" +
                "\t    \"type\": \"view\"\n" +
                "        },\n" +
                "        {\n" +
                "\t    \"name\": \"远程诊断\",\n" +
                "\t    \"url\": \"http://wechat.pacsonline.cn/wx_patient/web/examList?appId=" + ApiConfigKit.getApiConfig().getAppId() +"\",\n" +
                "\t    \"type\": \"view\"\n" +
                "        }\n" +
                "    ]\n" +
                "        },\n" +
                "        {\n" +
                "\t    \"name\": \"关于我们\",\n" +
                "\t    \"url\": \"http://wechat.pacsonline.cn/wx_patient/web/aboutUs\",\n" +
                "\t    \"type\": \"view\"\n" +
                "        }\n" +
                "    ]\n" +
                "}";
        ApiResult apiResult = MenuApi.createMenu(str);
        if (apiResult.isSucceed())
            renderText(apiResult.getJson());
        else
            renderText(apiResult.getErrorMsg());
    }

    /**
     * 获取公众号关注用户
     */
    public void getFollowers()
    {
        ApiResult apiResult = UserApi.getFollows();
        renderText(apiResult.getJson());
    }

    /**
     * 获取用户信息
     */
    public void getUserInfo()
    {
        ApiResult apiResult = UserApi.getUserInfo("ohbweuNYB_heu_buiBWZtwgi4xzU");
        renderText(apiResult.getJson());
    }

    /**
     * 发送模板消息
     */
    public void sendMsg()
    {
        String str = " {\n" +
                "           \"touser\":\"ohbweuNYB_heu_buiBWZtwgi4xzU\",\n" +
                "           \"template_id\":\"9SIa8ph1403NEM3qk3z9-go-p4kBMeh-HGepQZVdA7w\",\n" +
                "           \"url\":\"http://www.sina.com\",\n" +
                "           \"topcolor\":\"#FF0000\",\n" +
                "           \"data\":{\n" +
                "                   \"first\": {\n" +
                "                       \"value\":\"恭喜你购买成功！\",\n" +
                "                       \"color\":\"#173177\"\n" +
                "                   },\n" +
                "                   \"keyword1\":{\n" +
                "                       \"value\":\"去哪儿网发的酒店红包（1个）\",\n" +
                "                       \"color\":\"#173177\"\n" +
                "                   },\n" +
                "                   \"keyword2\":{\n" +
                "                       \"value\":\"1元\",\n" +
                "                       \"color\":\"#173177\"\n" +
                "                   },\n" +
                "                   \"remark\":{\n" +
                "                       \"value\":\"欢迎再次购买！\",\n" +
                "                       \"color\":\"#173177\"\n" +
                "                   }\n" +
                "           }\n" +
                "       }";
        ApiResult apiResult = TemplateMsgApi.send(str);
        renderText(apiResult.getJson());
    }
    
    /**
     * 统一获取微信二维码
     */
    public void unifyGetWxQrcode()
    {
        String reportId = getPara("reportId");
		Report reports = Report.dao.findById(reportId);
		Exams exams = Exams.dao.findById(reports.getExamId());
		Institution institution = Institution.dao.findById(exams.getInstitutionId());
		String appId;
		switch (institution.getWxSource()) {
		case "中世康恺电子胶片平台":
			appId = PropKit.get("appId_zskk");
			break;

		case "黑龙江省电子胶片平台":
			appId = PropKit.get("appId_hlj");
			break;

		case "辽宁省电子胶片平台":
			appId = PropKit.get("appId_ln");
			break;
			
		case "河南省电子胶片平台":
			appId = PropKit.get("appId_hn");
			break;

		default:
			appId = PropKit.get("appId_zskk");
			break;
		}
//		redirect("http://wechat.pacsonline.cn/wx_patient/api/getQrcode?appId=" + appId + "&reportId=" + reportId);
        ApiConfigKit.setThreadLocalAppId(appId);
        getQrcode(reportId);
    }
    
    /**
     * 获取参数二维码
     */
    public void getQrcode(String reportId)
    {
//        String reportId = getPara("reportId");
		Report reports = Report.dao.findById(reportId);
		if (reportId == null || reports == null) {
			renderNull();
			return;
		}
		if (reports.getQrCode() == null || reports.getQrCode().isEmpty()) {		
			String str = "{\"expire_seconds\": 2592000, \"action_name\": \"QR_STR_SCENE\", \"action_info\": {\"scene\": {\"scene_str\": \"ExamQR_" + reports.getExamId() + "\"}}}";
	        ApiResult apiResult = QrcodeApi.create(str);
	        apiResult.getStr("url");
			reports.setQrCode(apiResult.getStr("url"));
			reports.update();
	        renderText(apiResult.getJson());
		}else {
			//ApiResult qrresult = new ApiResult(reports.getQrCode());
			renderText(reports.getQrCode());;
		}
    }

    /**
     * 统一支付
     */
    public void unifyPay()
    {
        String reportId = getPara("reportId");
		Report reports = Report.dao.findById(reportId);
		Exams exams = Exams.dao.findById(reports.getExamId());
		Institution institution = Institution.dao.findById(exams.getInstitutionId());
		String appId;
		switch (institution.getWxSource()) {
		case "中世康恺电子胶片平台":
			appId = PropKit.get("appId_zskk");
			break;

		case "黑龙江省电子胶片平台":
			appId = PropKit.get("appId_hlj");
			break;

		case "辽宁省电子胶片平台":
			appId = PropKit.get("appId_ln");
			break;
			
		case "河南省电子胶片平台":
			appId = PropKit.get("appId_hn");
			break;

		default:
			appId = PropKit.get("appId_zskk");
			break;
		}
//		redirect("http://wechat.pacsonline.cn/wx_patient/api/getQrcode?appId=" + appId + "&reportId=" + reportId);
        ApiConfigKit.setThreadLocalAppId(appId);
//        String string = PaymentApi.pushOrder(j);
    }
    
    /*/
     * 获取JS-SDK的配置信息
     */
    public WxJSConfigBean getJsSdkConfig() 
    {
    	String origin = this.getRequest().getHeader("Origin");
		this.getResponse().setHeader("Access-Control-Allow-Origin", origin);
		this.getResponse().setHeader("Access-Control-Allow-Credentials", Boolean.TRUE.toString());
    	WeixinService wService = ServiceFactory.getService(WeixinService.class);
    	WxJSConfigBean wxJSConfigBean = wService.getWeixinJSSDKConfig(this.getRequest().getHeader("Referer"));
    	return wxJSConfigBean;
	}

	
    /**
     * 长链接转成短链接
     */
    public void getShorturl()
    {
        String str = "{\"action\":\"long2short\"," +
                "\"long_url\":\"http://wap.koudaitong.com/v2/showcase/goods?alias=128wi9shh&spm=h56083&redirect_count=1\"}";
        ApiResult apiResult = ShorturlApi.getShorturl(str);
        renderText(apiResult.getJson());
    }

    /**
     * 获取客服聊天记录
     */
    public void getRecord()
    {
        String str = "{\n" +
                "    \"endtime\" : 987654321,\n" +
                "    \"pageindex\" : 1,\n" +
                "    \"pagesize\" : 10,\n" +
                "    \"starttime\" : 123456789\n" +
                " }";
        ApiResult apiResult = CustomServiceApi.getRecord(str);
        renderText(apiResult.getJson());
    }

    /**
     * 获取微信服务器IP地址
     */
    public void getCallbackIp()
    {
        ApiResult apiResult = CallbackIpApi.getCallbackIp();
        renderText(apiResult.getJson());
    }
    
}

