package com.zskk.service;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.UUID;
import com.jfinal.kit.HashKit;
import com.jfinal.weixin.sdk.api.ApiConfigKit;
import com.jfinal.weixin.sdk.api.ApiResult;
import com.jfinal.weixin.sdk.api.JsTicket;
import com.jfinal.weixin.sdk.api.JsTicketApi;
import com.jfinal.weixin.sdk.api.SnsAccessToken;
import com.jfinal.weixin.sdk.api.SnsAccessTokenApi;
import com.jfinal.weixin.sdk.api.SnsApi;
import com.jfinal.weixin.sdk.api.JsTicketApi.JsApiType;
import com.zskk.model.User;
import com.zskk.service.bean.WxJSConfigBean;

public class WeixinService {
	
	/*/
	 * 微信消息推送类型枚举
	 */
	public enum PUSH_TYPE {
		
		//远程诊断患者支付推送
		REMOTE_DIAGNOSIS_PATIENT_PAY(1);
		
		private int pushType;
		
		PUSH_TYPE(int pushType) {
			this.pushType = pushType;
		}
		
		public int getPushType() {
			return pushType;
		}
		
	}
	
	/**
	 * 用户同意授权，获取带code的url
	 * @return
	 * @throws UnsupportedEncodingException 
	 */
	public String getCodeUrl(String appId, String redirect_uri) throws UnsupportedEncodingException {
		String encode = URLEncoder.encode(redirect_uri, "UTF-8");
		String url = SnsAccessTokenApi.getAuthorizeURL(appId, encode, false);
		return url;
	}

	/**
	 * 通过回调地址参数code获取access_token
	 * @param appId
	 * @param secret
	 * @param code
	 * @return
	 */
	public SnsAccessToken getWebAccessToken(String appId, String secret, String code) {
		SnsAccessToken snsAccessToken = SnsAccessTokenApi.getSnsAccessToken(appId, secret, code);
		return snsAccessToken;
	}
	
	/**
	 * 获取JsTicket，并生成JS-SDK调用参数
	 * @param requstUrl
	 * @return
	 */
	public WxJSConfigBean getWeixinJSSDKConfig(String requstUrl) {
		JsTicket jsTicket = JsTicketApi.getTicket(JsApiType.jsapi);
		String jsapi_ticket  = jsTicket.getTicket();
		String nonce_str 	 = create_nonce_str();
		String url 		     = requstUrl;
		String timestamp 	 = create_timestamp();
		String str 			 = "jsapi_ticket=" + jsapi_ticket + "&noncestr=" + nonce_str + "&timestamp=" + timestamp + "&url=" + url;
		String signature 	 = HashKit.sha1(str);
		
    	WxJSConfigBean wxJSConfigBean = new WxJSConfigBean();
    	wxJSConfigBean.setAppId(ApiConfigKit.getAppId());
    	wxJSConfigBean.setNonceStr(nonce_str);
    	wxJSConfigBean.setTimestamp(timestamp);
    	wxJSConfigBean.setSignature(signature);
		return wxJSConfigBean;
	}
    
	private String create_nonce_str() {
		return UUID.randomUUID().toString();
	}
	
	private static String create_timestamp() {
		return Long.toString(System.currentTimeMillis() / 1000);
	}
	/**
	 * 拉取用户信息
	 * @param accessToken
	 * @param openId
	 * @return
	 */
	public ApiResult getWebWeixinUserInfo(String accessToken, String openId) {
		ApiResult apiResult = SnsApi.getUserInfo(accessToken, openId);
		User user = User.dao.findFirst("select * from user where wx_openid=?",openId);
		user.setWxIcon(apiResult.getStr("headimgurl"));
		user.setWxNickname(apiResult.getStr("nickname"));
		user.update();
		return apiResult;
	}
	
	/*
	 * 发送模板消息
	 */
	public void sendTemplateMsg(String openid, String url, String event, String userName) {
//		if (PropKit.getBoolean("devMode") == true) {
//			ApiConfigKit.setThreadLocalApiConfig(getApiConfig());
//			SimpleDateFormat sdf = new SimpleDateFormat("yyyy年MM月dd日  HH:mm:ss");
//			String time = sdf.format(new Date());
//			String json = TemplateData.New().setTouser(openid)
//					.setTemplate_id("eNs4B9rKo_0N-sF1NRmW6PESmTLm8zi_5Vl6ujeFngA").setTopcolor("#743A3A")
//					.setUrl(url)
//					.add("keyword1", time  , "#743A3A")
//					.add("keyword2", event , "#743A3A")
//					.add("keyword3", "用药了吗？用药后感觉怎么样？快来说说吧，我们不会将您的资料泄露，完成赢积分！", "#743A3A")
//					.build();
//			System.out.println(json);
//			ApiResult result = TemplateMsgApi.send(json);
//			System.out.println(result.getJson());
//		}else{
//			ApiConfigKit.setThreadLocalApiConfig(getApiConfig());
//			SimpleDateFormat sdf = new SimpleDateFormat("yyyy年MM月dd日  HH:mm:ss");
//			String time = sdf.format(new Date());
//			String json = TemplateData.New().setTouser(openid)
//					.setTemplate_id("U4cjem6T4heIpO6gS92XE2au9ZTr0mJcgutgEoDIpRI").setTopcolor("#743A3A")
//					.setUrl(url)
//					.add("first"   , event      , "#0DB0EB")
//					.add("keyword1", userName         , "")
//					.add("keyword2", ""         , "")
//					.add("keyword3", time       , "")
//					.add("remark"  , "请按时记录哦", "#BBBBBB")
//					.build();
//			System.out.println(json);
//			ApiResult result = TemplateMsgApi.send(json);
//			System.out.println(result.getJson());
//		}
	}
	
	public static void main(String[] args) {
//			DruidPlugin druidPlugin = new DruidPlugin("jdbc:mysql://www.pacsonline.cn:3306/pacsonline?useSSL=false", "pacs",
//					"ZSKK@2017~!@#".trim());
//			druidPlugin.start();
//			ActiveRecordPlugin arp = new ActiveRecordPlugin(druidPlugin);
//
//			_MappingKit.mapping(arp);
//			arp.start();
//
//			Report report = Report.dao.findById("9c11936e4a8440d6");
		PUSH_TYPE dType2 = null;
		 for(PUSH_TYPE xxxEnum : PUSH_TYPE.values()){
	            if(xxxEnum.getPushType() == 1){
	    			dType2 = xxxEnum;
	    			System.out.println(xxxEnum);

	            }
	        }
		 switch (dType2) {
		case REMOTE_DIAGNOSIS_PATIENT_PAY:
			System.out.println("sdas");

			break;

		default:
			break;
		}
		PUSH_TYPE dType = PUSH_TYPE.REMOTE_DIAGNOSIS_PATIENT_PAY;
//			System.out.println(dType.getPushType());
		
	}
}
